/****************************************************************************
**
** DCOP Skeleton generated by dcopidl2cpp from kchmdcopiface.kidl
**
** WARNING! All changes made in this file will be lost!
**
*****************************************************************************/

#include "./kchmdcopiface.h"

#include <kdatastream.h>


static const char* const KCHMDCOPIface_ftable[6][3] = {
    { "void", "loadHelpFile(QString,QString)", "loadHelpFile(QString filename,QString url)" },
    { "void", "openPage(QString)", "openPage(QString url)" },
    { "void", "guiFindInIndex(QString)", "guiFindInIndex(QString word)" },
    { "void", "guiSearchQuery(QString)", "guiSearchQuery(QString query)" },
    { "QStringList", "searchQuery(QString)", "searchQuery(QString query)" },
    { 0, 0, 0 }
};
static const int KCHMDCOPIface_ftable_hiddens[5] = {
    0,
    0,
    0,
    0,
    0,
};

bool KCHMDCOPIface::process(const QCString &fun, const QByteArray &data, QCString& replyType, QByteArray &replyData)
{
    if ( fun == KCHMDCOPIface_ftable[0][1] ) { // void loadHelpFile(QString,QString)
	QString arg0;
	QString arg1;
	QDataStream arg( data, IO_ReadOnly );
	if (arg.atEnd()) return false;
	arg >> arg0;
	if (arg.atEnd()) return false;
	arg >> arg1;
	replyType = KCHMDCOPIface_ftable[0][0]; 
	loadHelpFile(arg0, arg1 );
    } else if ( fun == KCHMDCOPIface_ftable[1][1] ) { // void openPage(QString)
	QString arg0;
	QDataStream arg( data, IO_ReadOnly );
	if (arg.atEnd()) return false;
	arg >> arg0;
	replyType = KCHMDCOPIface_ftable[1][0]; 
	openPage(arg0 );
    } else if ( fun == KCHMDCOPIface_ftable[2][1] ) { // void guiFindInIndex(QString)
	QString arg0;
	QDataStream arg( data, IO_ReadOnly );
	if (arg.atEnd()) return false;
	arg >> arg0;
	replyType = KCHMDCOPIface_ftable[2][0]; 
	guiFindInIndex(arg0 );
    } else if ( fun == KCHMDCOPIface_ftable[3][1] ) { // void guiSearchQuery(QString)
	QString arg0;
	QDataStream arg( data, IO_ReadOnly );
	if (arg.atEnd()) return false;
	arg >> arg0;
	replyType = KCHMDCOPIface_ftable[3][0]; 
	guiSearchQuery(arg0 );
    } else if ( fun == KCHMDCOPIface_ftable[4][1] ) { // QStringList searchQuery(QString)
	QString arg0;
	QDataStream arg( data, IO_ReadOnly );
	if (arg.atEnd()) return false;
	arg >> arg0;
	replyType = KCHMDCOPIface_ftable[4][0]; 
	QDataStream _replyStream( replyData, IO_WriteOnly );
	_replyStream << searchQuery(arg0 );
    } else {
	return DCOPObject::process( fun, data, replyType, replyData );
    }
    return true;
}

QCStringList KCHMDCOPIface::interfaces()
{
    QCStringList ifaces = DCOPObject::interfaces();
    ifaces += "KCHMDCOPIface";
    return ifaces;
}

QCStringList KCHMDCOPIface::functions()
{
    QCStringList funcs = DCOPObject::functions();
    for ( int i = 0; KCHMDCOPIface_ftable[i][2]; i++ ) {
	if (KCHMDCOPIface_ftable_hiddens[i])
	    continue;
	QCString func = KCHMDCOPIface_ftable[i][0];
	func += ' ';
	func += KCHMDCOPIface_ftable[i][2];
	funcs << func;
    }
    return funcs;
}


